/* ***********************IMPORTANT *********************************************
 *  Copyright (C) 2004-2006 Thierry Lagarde <tlagarde@users.sourceforge.net>
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *
 *                      ********************
 *
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *
 *                      ********************
 *
 *
 * You are highly encouraged to send your changes to <tlagarde@users.sourceforge.net>
 * for possible incorporation into the main distribution.
 * By sending these changes to <tlagarde@users.sourceforge.net>,
 * it is assumed that you are offering <tlagarde@users.sourceforge.net>
 * the unlimited, non-exclusive right to reuse, modify, and relicense the code.
 *
 *
 *                      ********************
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 **************************************************************************************/
#include "Contribution.h"
#include "Debug.h"
#include "Declaration_Constante.h"
#include "Equipement.h"
#include "Equipement_Port.h"
#include "Interface/Combo.h"
#include "Interface/Fingerprinting_Interface.h"

gboolean Transfert_html (gchar * uri)
{
	GnomeVFSHandle *handle = NULL;
	GnomeVFSResult result;
	gchar     buffer[BUF_SIZE];
	GnomeVFSFileSize bytes_read;

	ERROR_if (uri == NULL);
	result = gnome_vfs_open (&handle, uri, GNOME_VFS_OPEN_READ);
	debug_printf (INTERFACE, D_ALL, __FILE__, __PRETTY_FUNCTION__, __LINE__, "result : %d", result);
	if (result != GNOME_VFS_OK) {
		debug_printf (INTERFACE, D_WARN_USER, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Unable to open URI, aborting\n");
		gnome_vfs_close (handle);
		return FALSE;
	}
	while (result == GNOME_VFS_OK) {
		result = gnome_vfs_read (handle, buffer, BUF_SIZE, &bytes_read);
		buffer[bytes_read] = 0;
		debug_printf (INTERFACE, D_ALL, __FILE__, __PRETTY_FUNCTION__, __LINE__, "(%d )%s", (int) bytes_read, buffer);
	}
	gnome_vfs_close (handle);
	return TRUE;
}


void contribution (gchar * Nom, gchar * Signature, gchar * Type, int Nb_Port_Ouvert, gchar * Adresse_MAC, gchar ftp, gchar http, gchar imap, gchar telnet, gchar spooler, gchar * EnteteFTP, gchar * EnteteHTTP, gchar * EnteteIMAP, gchar * EnteteTELNET, gchar * DebugSignature)
{
	gchar    *Contrib = NULL, *Nb_Port = NULL, *dEnteteFTP, *dEnteteHTTP, *dEnteteIMAP, *dEnteteTELNET;
	int       i;

	ERROR_if (Nom == NULL);
	ERROR_if (Signature == NULL);
	ERROR_if (Type == NULL);
	ERROR_if (Adresse_MAC == NULL);
	ERROR_if (EnteteFTP == NULL);
	ERROR_if (EnteteHTTP == NULL);
	ERROR_if (EnteteIMAP == NULL);
	ERROR_if (EnteteTELNET == NULL);
	ERROR_if (DebugSignature == NULL);

	dEnteteFTP = url_encode (EnteteFTP);	
	dEnteteHTTP = url_encode (EnteteHTTP);
	dEnteteIMAP = url_encode (EnteteIMAP);
	dEnteteTELNET = url_encode (EnteteTELNET);
	Nb_Port = g_strdup_printf ("%d", Nb_Port_Ouvert);
	Contrib = g_strdup_printf ("http://autoscan.free.fr/Add_Contributing_V6.php?Version=" VERSION_AUTOSCAN "&System_Name=%s&MD5_signature=%s&Type=%s&NB_PORT_OUVERT=%d&ADRESSE_MAC=%s&PORT_FTP=%d&PORT_HTTP=%d&PORT_IMAP=%d&PORT_TELNET=%d&PORT_SPOOLER=%d&ENTETE_FTP=[%s]&ENTETE_HTTP=[%s]&ENTETE_IMAP=[%s]&ENTETE_TELNET=[%s]&DEBUG_SIGNATURE=%s", Nom, Signature, Type, Nb_Port_Ouvert, Adresse_MAC, ftp, http, imap, telnet, spooler, dEnteteFTP, dEnteteHTTP, dEnteteIMAP, dEnteteTELNET, DebugSignature);
	g_free (dEnteteFTP);
	g_free (dEnteteHTTP);
	g_free (dEnteteIMAP);
	g_free (dEnteteTELNET);
	
	for (i = 0; i < strlen (Contrib); i++) {
		if (Contrib[i] == ' ')
			Contrib[i] = '_';
	}
	debug_printf (INTERFACE, D_ALL, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Web : (%s)\n", Contrib);
	if (!Transfert_html (Contrib))
		debug_printf (INTERFACE, D_WARN_USER, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Error send\n");
	g_free (Contrib);
	g_free (Nb_Port);
}



gpointer Thread_Contribution_Fingerprinting ()
{
	char     *debug, *combo;

	debug = url_encode (Equipement_Selectionner->Debug_Signature);
	combo = Recuperation_Texte_Combo (combobox_Information_OS_Type);
	Etat_Contribution = CONTRIBUTION_DONNEE_OK;
	contribution ((gchar *) gtk_entry_get_text (GTK_ENTRY (entry_Information_OS_System_Name)), Equipement_Selectionner->Md5_Signature, combo, Equipement_Selectionner->Nb_Port_Ouvert, Equipement_Selectionner->Adresse_MAC_Char, Cherche_Si_Port_Ouvert (Equipement_Selectionner, PORT_FTP), (char) Cherche_Si_Port_Ouvert (Equipement_Selectionner, PORT_HTTP), (char) Cherche_Si_Port_Ouvert (Equipement_Selectionner, PORT_IMAP), (char) Cherche_Si_Port_Ouvert (Equipement_Selectionner, PORT_TELNET), (char) Cherche_Si_Port_Ouvert (Equipement_Selectionner, PORT_SPOOLER), Equipement_Selectionner->Entete_FTP, Equipement_Selectionner->Entete_HTTP, Equipement_Selectionner->Entete_IMAP, Equipement_Selectionner->Entete_TELNET, debug);
	free (debug);
	g_free (combo);		
	debug_printf (INTERFACE, D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Fin Contribution");
	g_thread_exit (0);
}




void contribution_Erreur (gchar * Message_Erreur)
{
	gchar    *Contrib = NULL, *Message_URL = NULL;
	int       i;

	ERROR_if (Message_Erreur == NULL);
	Message_URL = url_encode (Message_Erreur);
	Contrib = g_strdup_printf ("http://autoscan.free.fr/Erreur.php?Version=" VERSION_AUTOSCAN "&Message_Erreur=%s", Message_URL);
	g_free (Message_URL);
	
	for (i = 0; i < strlen (Contrib); i++) {
		if (Contrib[i] == ' ')
			Contrib[i] = '_';
	}
	if (!Transfert_html (Contrib))
		debug_printf (INTERFACE, D_WARN_USER, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Error send\n");
	g_free (Contrib);
}
