/* ***********************IMPORTANT *********************************************
 *  Copyright (C) 2004-2006 Thierry Lagarde <tlagarde@users.sourceforge.net>
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *
 *                      ********************
 *
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *
 *                      ********************
 *
 *
 * You are highly encouraged to send your changes to <tlagarde@users.sourceforge.net>
 * for possible incorporation into the main distribution.
 * By sending these changes to <tlagarde@users.sourceforge.net>,
 * it is assumed that you are offering <tlagarde@users.sourceforge.net>
 * the unlimited, non-exclusive right to reuse, modify, and relicense the code.
 *
 *
 *                      ********************
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 **************************************************************************************/
#include "Enregistrement.h"
#include "Equipement.h"
#include "Equipement_Port.h"
#include "Equipement_Historique.h"
#include "Add_Network_Subnet.h"
#include "Reseau.h"
#include "Debug.h"
#include "Objet.h"
#include "Snmp.h"

static void Write_xml_Port_Equipement (xmlNodePtr xmlnode, struct Struct_Port *Port)
{
	xmlNodePtr newxml = NULL;
	gchar    *Port_Char = NULL;
	gchar     Date[30];

	ERROR_if (xmlnode == NULL);
	ERROR_if (Port == NULL);
	newxml = xmlNewChild (xmlnode, NULL, "Port", NULL);
	Port_Char = g_strdup_printf ("%d", Port->Num_Port);
	xmlSetProp (newxml, "Num", Port_Char);
	g_free (Port_Char);
	if (Port->Notification_En_Cours)
		xmlSetProp (newxml, "Notification", "1");
	else
		xmlSetProp (newxml, "Notification", "0");
	ctime_r (&Port->Derniere_Modification_Etat_Port, Date);
	Date[strlen (Date) - 1] = '\0';	
	xmlSetProp (newxml, "Last_State_Change", Date);
}


static void Write_xml_Signature_Equipement (xmlNodePtr xmlnode, struct Struct_Equipement *Equipement)
{
	xmlNodePtr newxml = NULL;

	ERROR_if (xmlnode == NULL);
	ERROR_if (Equipement == NULL);
	newxml = xmlNewChild (xmlnode, NULL, "FingerPrinting", NULL);
	xmlSetProp (newxml, "MD5", Equipement->Md5_Signature);
	xmlSetProp (newxml, "Header_FTP", Equipement->Entete_FTP);
	xmlSetProp (newxml, "Header_HTTP", Equipement->Entete_HTTP);
	xmlSetProp (newxml, "Header_IMAP", Equipement->Entete_IMAP);
	xmlSetProp (newxml, "Header_TELNET", Equipement->Entete_TELNET);
	xmlSetProp (newxml, "Debug", Equipement->Debug_Signature);
}


static void Write_xml_Historique_Equipement (xmlNodePtr xmlnode, struct Struct_Historique *Historique)
{
	xmlNodePtr newxml = NULL;
	gchar    *Niveau_Historique = NULL;
	gchar     Date[30];

	ERROR_if (xmlnode == NULL);
	ERROR_if (Historique == NULL);
	newxml = xmlNewChild (xmlnode, NULL, "History", NULL);
	ctime_r (&Historique->Date, Date);
	Date[strlen (Date) - 1] = '\0';	
	xmlSetProp (newxml, "Date", Date);
	Niveau_Historique = g_strdup_printf ("%d", Historique->Niveau_Historique);
	xmlSetProp (newxml, "Level", Niveau_Historique);
	g_free (Niveau_Historique);
	xmlSetProp (newxml, "Text", Historique->Historique);
}


static void Write_xml_Nessus_Equipement (xmlNodePtr xmlnode, struct Struct_Nessus *Nessus)
{
	xmlNodePtr newxml = NULL;
	gchar    *Niveau_Nessus = NULL;

	ERROR_if (xmlnode == NULL);
	ERROR_if (Nessus == NULL);
	newxml = xmlNewChild (xmlnode, NULL, "Nessus", NULL);
	Niveau_Nessus = g_strdup_printf ("%d", Nessus->Niveau_Nessus);
	xmlSetProp (newxml, "Level", Niveau_Nessus);
	g_free (Niveau_Nessus);
	xmlSetProp (newxml, "Info", Nessus->Info);
	xmlSetProp (newxml, "Text", Nessus->Texte);
}


static void Write_xml_Samba_Equipement (xmlNodePtr xmlnode, struct Struct_Equipement *Equipement, gint i)
{
	xmlNodePtr newxml = NULL;
	gchar    *tmp = NULL;

	ERROR_if (xmlnode == NULL);
	ERROR_if (Equipement == NULL);
	newxml = xmlNewChild (xmlnode, NULL, "Registration", NULL);
	tmp = g_strdup_printf ("%d", Equipement->Smb.Smb_Registration[i].Numero);
	xmlSetProp (newxml, "Number", tmp);
	g_free (tmp);
	tmp = g_strdup_printf ("%d", Equipement->Smb.Smb_Registration[i].Type);
	xmlSetProp (newxml, "Type", tmp);
	g_free (tmp);
	xmlSetProp (newxml, "Name", Equipement->Smb.Smb_Registration[i].Name);
}


static void Write_xml_Subnet (xmlNodePtr xmlnode, struct Struct_Subnet *Subnet)
{
	xmlNodePtr newxml = NULL;
	gchar    *tmp = NULL;

	ERROR_if (xmlnode == NULL);
	ERROR_if (Subnet == NULL);
	newxml = xmlNewChild (xmlnode, NULL, "Subnet", NULL);
	tmp = g_strdup_printf ("%d", Subnet->Masque);
	xmlSetProp (newxml, "Masque", tmp);
	g_free (tmp);
	tmp = g_strdup_printf ("%d", Subnet->A);
	xmlSetProp (newxml, "A", tmp);
	g_free (tmp);
	tmp = g_strdup_printf ("%d", Subnet->B);
	xmlSetProp (newxml, "B", tmp);
	g_free (tmp);	
	tmp = g_strdup_printf ("%d", Subnet->C);
	xmlSetProp (newxml, "C", tmp);
	g_free (tmp);
}


static void Write_xml (xmlNodePtr xmlnode, struct Struct_Equipement *Equipement)
{
	xmlNodePtr newxml = NULL, newxml_Rep = NULL;
	struct Struct_Port *Port = NULL;
	struct Struct_Historique *Historique = NULL;
	struct Struct_Nessus *Nessus = NULL;
	
	gchar    *tmp = NULL;
	gchar     Date[30];
	guint     Nb_Objet_Liste, i;

	ERROR_if (xmlnode == NULL);
	ERROR_if (Equipement == NULL);
	newxml = xmlNewChild (xmlnode, NULL, "Equipment", NULL);
	
	xmlSetProp (newxml, "Address_IP", Equipement->Adresse_Ip_Char);
	xmlSetProp (newxml, "Address_MAC", Equipement->Adresse_MAC_Char);
	xmlSetProp (newxml, "Localisation", Equipement->Localisation);
	xmlSetProp (newxml, "Admin_Email", Equipement->Contact_Admin);

	xmlSetProp (newxml, "Serial_Number", Equipement->Num_Serie);
	xmlSetProp (newxml, "Serial_Number_Display", Equipement->Num_Serie_Ecran);
	xmlSetProp (newxml, "Serial_Number_Printer", Equipement->Num_Serie_Imprimante);
	xmlSetProp (newxml, "Serial_Number_OS", Equipement->Num_Serie_OS);
	xmlSetProp (newxml, "Serial_Number_Office_Suite", Equipement->Num_Serie_Office_Suite);
	xmlSetProp (newxml, "Processor", Equipement->Processeur);
	xmlSetProp (newxml, "Memory", Equipement->Memoire);
	xmlSetProp (newxml, "Disk", Equipement->Disque);
	xmlSetProp (newxml, "Contact", Equipement->Contact);
	xmlSetProp (newxml, "Tel_Contact", Equipement->Tel_Contact);
	xmlSetProp (newxml, "Notes", Equipement->Notes);

	if (Equipement->Agent_AutoScan)
		xmlSetProp (newxml, "Agent_AutoScan", "1");
	else
		xmlSetProp (newxml, "Agent_AutoScan", "0");

	if (Equipement->Notification_En_Cours)
		xmlSetProp (newxml, "Notification", "1");
	else
		xmlSetProp (newxml, "Notification", "0");
	ctime_r (&Equipement->Derniere_Modification_Etat_Equipement, Date);
	Date[strlen (Date) - 1] = '\0';	
	xmlSetProp (newxml, "Last_State_Change", Date);
	
	Write_xml_Signature_Equipement (newxml, Equipement);
	
	newxml_Rep = xmlNewChild (newxml, NULL, "Ports", NULL);
	tmp = g_strdup_printf ("%d", Equipement->Nb_Port_Ouvert);
	xmlSetProp (newxml_Rep, "Open", tmp);
	g_free (tmp);
	tmp = g_strdup_printf ("%d", Equipement->Nb_Port_Total_Scanner);
	xmlSetProp (newxml_Rep, "Total", tmp);
	g_free (tmp);
	tmp = g_strdup_printf ("%d", Equipement->Nb_Port_Firewaller);
	xmlSetProp (newxml_Rep, "firewalled", tmp);
	g_free (tmp);
	
	Nb_Objet_Liste = g_list_length (Equipement->GList_Port);
	for (i = 0; i < Nb_Objet_Liste; i++) {
		Port = g_list_nth_data (Equipement->GList_Port, i);
		Write_xml_Port_Equipement (newxml_Rep, Port);
	}
	
	newxml_Rep = xmlNewChild (newxml, NULL, "History", NULL);
	Nb_Objet_Liste = g_list_length (Equipement->GList_Historique);
	for (i = 0; i < Nb_Objet_Liste; i++) {
		Historique = g_list_nth_data (Equipement->GList_Historique, i);
		Write_xml_Historique_Equipement (newxml_Rep, Historique);
	}
	
	newxml_Rep = xmlNewChild (newxml, NULL, "Smb", NULL);
	xmlSetProp (newxml_Rep, "Host_Name", Equipement->Smb.HostName);
	xmlSetProp (newxml_Rep, "Domain_Name", Equipement->Smb.DomainName);
	xmlSetProp (newxml_Rep, "User", Equipement->Smb.Utilisateur);
	xmlSetProp (newxml_Rep, "User_Smb", Equipement->User_Smb);
	xmlSetProp (newxml_Rep, "Password_Smb", Equipement->Password_Smb);
	
	i = 0;
	while ((i < NUM_MAX_REGISTRATION) && (Equipement->Smb.Smb_Registration[i].Type != 0)) {
		Write_xml_Samba_Equipement (newxml_Rep, Equipement, i);
		i++;
	}
	
	newxml_Rep = xmlNewChild (newxml, NULL, "Nessus", NULL);
	Nb_Objet_Liste = g_list_length (Equipement->GList_Nessus);
	for (i = 0; i < Nb_Objet_Liste; i++) {
		Nessus = g_list_nth_data (Equipement->GList_Nessus, i);
		Write_xml_Nessus_Equipement (newxml_Rep, Nessus);
	}
	
}


static gboolean write_xml (char *fname)
{
	xmlDocPtr doc = NULL;
	struct Struct_Equipement *Equipement = NULL;
	struct Struct_Modele_Objet *Objet = NULL;
	struct Struct_Subnet *Subnet;
	guint     Nb_Objet_Liste, i;
	xmlNodePtr newxml_Rep = NULL;
	xmlNode *root_element = NULL;
	
	ERROR_if (fname == NULL);
	
	
	
	
	doc = xmlNewDoc(BAD_CAST "1.0");
    root_element = xmlNewNode(NULL, BAD_CAST "AutoScan");
    xmlDocSetRootElement(doc, root_element);
	
	
	xmlAddChild (root_element, xmlNewDocComment (doc, "Copyright (c) Lagarde thierry. \nThis Database is available for free use by open source software under the terms of the GNU General Public License.\nContact <tlagarde@users.sourceforge.net>  for more information"));
	
	
	
	xmlSetProp (root_element, "Network", Reseau_Selectionner->Nom_Reseau);
	xmlSetProp (root_element, "Interface", Reseau_Selectionner->Interface);
	xmlSetProp (root_element, "Ip", Reseau_Selectionner->Ip);
	xmlSetProp (root_element, "Communaute_Snmp", Reseau_Selectionner->Communaute_Snmp);
	xmlSetProp (root_element, "Icon",Reseau_Selectionner->Icone_Reseau);
	if (Reseau_Selectionner->Alert_Intrusion)
		xmlSetProp (root_element, "Intrusion_Alert", "1");
	else
		xmlSetProp (root_element, "Intrusion_Alert", "0");
	
	if (Reseau_Selectionner->Mode_Liste_Detaille)
		xmlSetProp (root_element, "Mode_IP", "1");
	else
		xmlSetProp (root_element, "Mode_IP", "0");
	
	newxml_Rep = xmlNewChild (root_element, NULL, "Subnet", NULL);
	Nb_Objet_Liste = g_list_length (Reseau_Selectionner->Glist_Subnet_Equipement);
	for (i = 0; i < Nb_Objet_Liste; i++) {
		Subnet= g_list_nth_data (Reseau_Selectionner->Glist_Subnet_Equipement, i);
		Write_xml_Subnet (newxml_Rep, Subnet);
	}
	
	
	Nb_Objet_Liste = g_list_length (Reseau_Selectionner->GList_Liste_Equipement);
	for (i = 0; i < Nb_Objet_Liste; i++) {
		
		Objet = g_list_nth_data (Reseau_Selectionner->GList_Liste_Equipement, i);
		if (Objet->Nb_Enfant == 0) {	
			Equipement = (struct Struct_Equipement *) Objet->Pointeur_Attacher;
			Write_xml (root_element, Equipement);
		}
	}
	if (xmlSaveFile (fname, doc) == -1) {
		xmlFreeDoc (doc);
		debug_printf (INTERFACE, D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "impossible d'enregistrer le fichier xml");
		return FALSE;
	}
	xmlFreeDoc (doc);
	return TRUE;
}




void Enregistrement (gchar * Chemin_Fichier)
{
	ERROR_if (Chemin_Fichier == NULL);
	debug_printf (INTERFACE, D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Save : [%s]", Chemin_Fichier);
	write_xml (Chemin_Fichier);
}
