/* ***********************IMPORTANT *********************************************
 *  Copyright (C) 2004-2006 Thierry Lagarde <tlagarde@users.sourceforge.net>
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *
 *                      ********************
 *
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *
 *                      ********************
 *
 *
 * You are highly encouraged to send your changes to <tlagarde@users.sourceforge.net>
 * for possible incorporation into the main distribution.
 * By sending these changes to <tlagarde@users.sourceforge.net>,
 * it is assumed that you are offering <tlagarde@users.sourceforge.net>
 * the unlimited, non-exclusive right to reuse, modify, and relicense the code.
 *
 *
 *                      ********************
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 **************************************************************************************/
#include <gnome.h>

#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <pwd.h>
#include <stdlib.h>

#include <gtk/gtk.h>
#include <gdk/gdkx.h>
#include <gdk/gdkprivate.h>
#include <gdk/gdkkeysyms.h>

#include <libzvt/libzvt.h>
#include "Root_Interface.h"
#include "Console.h"
#include "../Modules.h"
#include "../Equipement.h"
#include "../Debug.h"

#define FONT "-misc-fixed-medium-r-normal--12-200-75-75-c-100-iso8859-1"	

extern char **environ;
static char **env;
static char **env_copy;
static int winid_pos;
static GtkWidget *window = NULL;

gushort   linux_red[] = { 0x0000, 0xaaaa, 0x0000, 0xaaaa, 0x0000, 0xaaaa, 0x0000, 0xaaaa,
	0x5555, 0xffff, 0x5555, 0xffff, 0x5555, 0xffff, 0x5555, 0xffff,
	0x0, 0x0
};
gushort   linux_grn[] = { 0x0000, 0x0000, 0xaaaa, 0x5555, 0x0000, 0x0000, 0xaaaa, 0xaaaa,
	0x5555, 0x5555, 0xffff, 0xffff, 0x5555, 0x5555, 0xffff, 0xffff,
	0x0, 0x0
};
gushort   linux_blu[] = { 0x0000, 0x0000, 0x0000, 0x0000, 0xaaaa, 0xaaaa, 0xaaaa, 0xaaaa,
	0x5555, 0x5555, 0x5555, 0x5555, 0xffff, 0xffff, 0xffff, 0xffff,
	0x0, 0x0
};


static void child_died_event (ZvtTerm * term)
{
	zvt_term_closepty (term);
	debug_printf (INTERFACE, D_ALL, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Fin telnet\n");
}


static void title_changed_event (ZvtTerm * term, VTTITLE_TYPE type, char *newtitle)
{
	switch (type) {
	    case VTTITLE_WINDOW:
	    case VTTITLE_WINDOWICON:
		    gtk_window_set_title (GTK_WINDOW (window), newtitle);
		    break;
	    default:
		    break;
	}
}


static int button_press_event (ZvtTerm * term, GdkEventButton * e)
{
	int       x, y;
	GdkModifierType mask;
	char     *data, *str;

	gtk_widget_grab_focus (GTK_WIDGET (term));	
	gdk_window_get_pointer (GTK_WIDGET (term)->window, &x, &y, &mask);
	str = zvt_term_match_check (term, x / term->charwidth, y / term->charheight, (void *) &data);
	if (str) {
		g_signal_stop_emission_by_name (term, "button_press_event");
		debug_printf (INTERFACE, D_ALL, __FILE__, __PRETTY_FUNCTION__, __LINE__, "User clicked on type '%s' -> '%s'\n", data, str);
		return TRUE;
	}
	return FALSE;
}


static void set_hints (GtkWidget * widget)
{
	ZvtTerm  *term;
	GdkGeometry hints;
	GtkWidget *app;

	g_assert (widget != NULL);
	term = ZVT_TERM (widget);
	app = gtk_widget_get_toplevel (widget);
	g_assert (app != NULL);
#define PADDING 2
	hints.base_width = (GTK_WIDGET (term)->style->xthickness * 2) + PADDING;
	hints.base_height = (GTK_WIDGET (term)->style->ythickness * 2);
	hints.width_inc = term->charwidth;
	hints.height_inc = term->charheight;
	hints.min_width = hints.base_width + hints.width_inc;
	hints.min_height = hints.base_height + hints.height_inc;
	gtk_window_set_geometry_hints (GTK_WINDOW (app), GTK_WIDGET (term), &hints, GDK_HINT_RESIZE_INC | GDK_HINT_MIN_SIZE | GDK_HINT_BASE_SIZE);
}


GtkWidget *Creation_Console (gchar * Commande)
{
	int       i, scrollbacklines, login_shell;
	char    **p;
	GtkWidget *term, *hbox, *scrollbar;

	login_shell = 0;
	scrollbacklines = 50;
	
	env = environ;
	for (p = env; *p; p++) ;
	i = p - env;
	env_copy = (char **) g_malloc (sizeof (char *) * (i + 3));
	for (i = 0, p = env; *p; p++) {
		if (strncmp (*p, "TERM=", 5) == 0)
			env_copy[i++] = "TERM=xterm";
		else if ((strncmp (*p, "COLUMNS=", 8) == 0) || (strncmp (*p, "LINES=", 6) == 0))
			continue;
		else
			env_copy[i++] = *p;
	}
	env_copy[i++] = "COLORTERM=zterm";
	winid_pos = i++;
	env_copy[winid_pos] = "TEST";
	env_copy[i] = NULL;
	
	hbox = gtk_hbox_new (FALSE, 0);
	gtk_box_set_spacing (GTK_BOX (hbox), 2);
	gtk_container_set_border_width (GTK_CONTAINER (hbox), 2);
	gtk_container_add (GTK_CONTAINER (viewport_Modules), hbox);
	gtk_widget_show (hbox);
	
	term = zvt_term_new_with_size (80, 24);
	zvt_term_set_font_name (ZVT_TERM (term), "Courier 10 Pitch 12");
	zvt_term_set_blink (ZVT_TERM (term), TRUE);
	zvt_term_set_bell (ZVT_TERM (term), TRUE);
	zvt_term_set_scrollback (ZVT_TERM (term), scrollbacklines);
	zvt_term_set_scroll_on_keystroke (ZVT_TERM (term), TRUE);
	zvt_term_set_scroll_on_output (ZVT_TERM (term), FALSE);
	
	
	zvt_term_set_wordclass (ZVT_TERM (term), "-A-Za-z0-9/_:.,?+%=");
	zvt_term_match_add (ZVT_TERM (term), "http://[^ ]*[^\\.]", VTATTR_UNDERLINE | (1 << VTATTR_FORECOLOURB) | (2 << VTATTR_BACKCOLOURB), "http");
	zvt_term_match_add (ZVT_TERM (term), "ftp://[^ ]*[^\\.]", VTATTR_BOLD, "ftp");
	zvt_term_match_add (ZVT_TERM (term), "file://[^ ]*[^\\.]", VTATTR_REVERSE, "file");
	zvt_term_match_add (ZVT_TERM (term), "[0-9\\.]*", VTATTR_BOLD, "ip #");
	g_signal_connect (G_OBJECT (term), "button_press_event", G_CALLBACK (button_press_event), NULL);
	g_signal_connect (G_OBJECT (term), "child_died", G_CALLBACK (child_died_event), NULL);
	g_signal_connect (G_OBJECT (term), "destroy", G_CALLBACK (child_died_event), NULL);
	g_signal_connect (G_OBJECT (term), "title_changed", G_CALLBACK (title_changed_event), NULL);
	g_signal_connect_after (G_OBJECT (term), "realize", G_CALLBACK (set_hints), term);
	gtk_widget_show (term);
	scrollbar = gtk_vscrollbar_new (GTK_ADJUSTMENT (ZVT_TERM (term)->adjustment));
	gtk_box_pack_start (GTK_BOX (hbox), term, 1, 1, 0);
	gtk_box_pack_start (GTK_BOX (hbox), scrollbar, FALSE, TRUE, 0);
	gtk_widget_show (scrollbar);
	gtk_widget_grab_focus (term);
	
	switch (zvt_term_forkpty (ZVT_TERM (term), ZVT_TERM_DO_UTMP_LOG | ZVT_TERM_DO_WTMP_LOG | ZVT_TERM_DO_LASTLOG)) {
	    case -1:
		    perror ("unable to fork:");
		    _exit (1);
		    break;
	    case 0:{
			    environ = env_copy;
			    execlp (Commande, Commande, Equipement_Selectionner->Adresse_Ip_Char, NULL);
		    }
	    default:
		    break;
	}
	return hbox;
}
