/* ***********************IMPORTANT *********************************************
 *  Copyright (C) 2004-2006 Thierry Lagarde <tlagarde@users.sourceforge.net>
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *
 *                      ********************
 *
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *
 *                      ********************
 *
 *
 * You are highly encouraged to send your changes to <tlagarde@users.sourceforge.net>
 * for possible incorporation into the main distribution.
 * By sending these changes to <tlagarde@users.sourceforge.net>,
 * it is assumed that you are offering <tlagarde@users.sourceforge.net>
 * the unlimited, non-exclusive right to reuse, modify, and relicense the code.
 *
 *
 *                      ********************
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 **************************************************************************************/
#include <gnome.h>
#include "Smtp.h"
#include "Debug.h"
#include "Gconf.h"
#include "Thread.h"

#define  DONNEE_PRET 1
#define  ATTENDRE    2
#define  ERREUR_SOCK 3



int waiting_service (int sock, int event)
{
	struct pollfd fds;
	int       resultat;

	bzero (&fds, sizeof fds);	
	fds.events = event;
	fds.fd = sock;		
	debug_printf (INTERFACE, D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "waiting_service : (%d)", sock);
	if ((resultat = poll (&fds, 1, 1000)) > 0) {	
		if ((fds.revents & POLLERR) == POLLERR) {
			debug_printf (INTERFACE, D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "waiting_service Socket error : (%d)", sock);
			return ERREUR_SOCK;
		}
		if ((fds.revents & POLLHUP) == POLLHUP) {
			debug_printf (INTERFACE, D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "waiting_service Socket error : (%d)", sock);
			return ERREUR_SOCK;
		}
		debug_printf (INTERFACE, D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "waiting_service : (%d) pret", sock);
		return DONNEE_PRET;
	}
	else {
		if (resultat != 0) {
			debug_printf (INTERFACE, D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "%s", strerror (errno));
		}
		debug_printf (INTERFACE, D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "waiting_service : (%d) Attendre", sock);
		return ATTENDRE;
	}
}


gboolean attendre_Sock (int sock)
{
	int       temp;
	int       result;

	temp = 5;
	while ((result = waiting_service (sock, POLLOUT)) == ATTENDRE) {
		printf (".%d\n", temp);
		temp--;
		if (temp == 0) {
			debug_printf (INTERFACE, D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Timeout dans Attendre_donnee");
			return FALSE;
		}
	}
	if (result == ERREUR_SOCK)
		return FALSE;
	return TRUE;
}

int connection_Serveur (char *serveur, struct hostent *host, int port, int sock)
{
	struct sockaddr_in SocketAddr;
	memset (&SocketAddr, 0, sizeof (struct sockaddr_in));
	memcpy (&SocketAddr.sin_addr.s_addr, host->h_addr, host->h_length);
	SocketAddr.sin_family = AF_INET;
	SocketAddr.sin_port = htons (port);
	debug_printf (INTERFACE, D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "[I] Connection au serveur [%s] en cours\n", serveur);
	if (connect (sock, (struct sockaddr *) &SocketAddr, sizeof (struct sockaddr)) < 0) {
		if (errno != EINPROGRESS) {
			debug_printf (INTERFACE, D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "%s", strerror (errno));
			return FALSE;
		}
		if (attendre_Sock (sock) == FALSE)
			return FALSE;
	}
	debug_printf (INTERFACE, D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Connection au serveur : %s:%i\n", serveur, port);
	return TRUE;
}





gboolean Envoie_Donnee (int Socket, char *Donnee, char *Reponse)
{
	int       NbOctet;
	char      Temp[255];

	if ((NbOctet = send (Socket, Donnee, strlen (Donnee), 0)) == 0) {
		debug_printf (INTERFACE, D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Mail send");
		close (Socket);
		return FALSE;
	}

	if ((NbOctet = recv (Socket, Temp, sizeof (Temp), 0)) == 0) {
		debug_printf (INTERFACE, D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Mail recv");
		close (Socket);
		return FALSE;
	}
	Temp[NbOctet] = 0;
	debug_printf (INTERFACE, D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Command : %s#  ---> [%s]\n", Donnee, Temp);
	if (strncmp (Temp, Reponse, strlen (Reponse)) != 0) {
		debug_printf (INTERFACE, D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Error SMTP");
		return FALSE;
	}
	return TRUE;
}


void Envoie_Email (gpointer * Donnee)
{
	struct struct_Smtp *Donnee_SMTP = (struct struct_Smtp *) Donnee;
	struct hostent *host;
	int       Socket;
	int       NbOctet, Tentative;
	char      Temp[255];
	gboolean  Connection_Accepter;


	debug_printf (INTERFACE, D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Email to %s", Lecture_Gconf_Char (CHAR_SERVEUR_SMTP));
	if ((host = gethostbyname (Lecture_Gconf_Char (CHAR_SERVEUR_SMTP))) == NULL) {
		g_thread_exit (0);
	}
	Socket = socket (AF_INET, SOCK_STREAM, 0);

	Tentative = 0;
	Connection_Accepter = FALSE;

	do {
		if (connection_Serveur (Lecture_Gconf_Char (CHAR_SERVEUR_SMTP), host, 25, Socket) == FALSE) {
			sleep (5);
		}
		else
			Connection_Accepter = TRUE;
		Tentative++;
	} while ((!Connection_Accepter) && (Tentative <= 5));

	if (!Connection_Accepter) {
		debug_printf (INTERFACE, D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Impossible de se connecter");
		close (Socket);
		g_thread_exit (0);
	}

	if ((NbOctet = recv (Socket, Temp, sizeof (Temp), 0)) == 0) {
		debug_printf (INTERFACE, D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Reception En-tete");
		close (Socket);
		g_thread_exit (0);
	}
	Temp[NbOctet] = 0;
	sprintf (Temp, "HELO %s\r\n", Lecture_Gconf_Char (CHAR_SERVEUR_SMTP));
	if (!Envoie_Donnee (Socket, Temp, "250"))
		g_thread_exit (0);

	sprintf (Temp, "MAIL FROM:<%s>\r\n",Donnee_SMTP->utilisateur);
	if (!Envoie_Donnee (Socket, Temp, "250"))
		g_thread_exit (0);
	sprintf (Temp, "RCPT TO:<%s>\r\n", Donnee_SMTP->utilisateur);
	if (!Envoie_Donnee (Socket, Temp, "250"))
		g_thread_exit (0);
	if (!Envoie_Donnee (Socket, "DATA\r\n", "354"))
		g_thread_exit (0);

	if ((NbOctet = send (Socket, Donnee_SMTP->Message, strlen (Donnee_SMTP->Message), 0)) == 0) {
		debug_printf (INTERFACE, D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Envoie Mail From Erreur\n");
		close (Socket);
		g_thread_exit (0);
	}
	if (!Envoie_Donnee (Socket, "\r\n.\r\n", "250"))
		g_thread_exit (0);
	if (!Envoie_Donnee (Socket, "QUIT\r\n", "221"))
		g_thread_exit (0);
	close (Socket);
	printf ("Email send\n");
	g_free (Donnee_SMTP->utilisateur);
	g_free (Donnee_SMTP->Message);
	g_free (Donnee_SMTP);
	g_thread_exit (0);
}



void Envoie_Email_Notification (gchar * Message, gchar * utilisateur)
{
	struct struct_Smtp *SMTP;

	SMTP = (struct struct_Smtp *) malloc (sizeof (struct struct_Smtp));
	ERROR_if (SMTP == NULL);
	bzero (SMTP, sizeof (struct struct_Smtp));
	SMTP->utilisateur = g_strdup (utilisateur);
	SMTP->Message = g_strdup_printf ("Subject: Notification AutoScan\n%s", Message);
	Creation_Thread ((gpointer) & Envoie_Email, SMTP);
}
