/* ***********************IMPORTANT *********************************************
 *  Copyright (C) 2004-2006 Thierry Lagarde <tlagarde@users.sourceforge.net>
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *
 *                      ********************
 *
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *
 *                      ********************
 *
 *
 * You are highly encouraged to send your changes to <tlagarde@users.sourceforge.net>
 * for possible incorporation into the main distribution.
 * By sending these changes to <tlagarde@users.sourceforge.net>,
 * it is assumed that you are offering <tlagarde@users.sourceforge.net>
 * the unlimited, non-exclusive right to reuse, modify, and relicense the code.
 *
 *
 *                      ********************
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 **************************************************************************************/
#include <gnome.h>
#include "Sound.h"
#include "Debug.h"
#include "Gconf.h"
#include "Thread.h"

gpointer Thread_Sound_Alert ()
{
	
	ao_device *device;	
	ao_sample_format format;	
	int       default_driver;	

	
	OggVorbis_File vf;
	vorbis_info *vi;
	long      ret;
	int       current_section;
	FILE     *f;

	
	char      pcmout[BUF_SIZE];	


	
	if ((f = fopen (Lecture_Gconf_Char (CHAR_CHEMIN_SON_ALERT), "r")) == NULL) {
		debug_printf (INTERFACE, D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Error opening redalert.ogg");
		g_thread_exit (0);
	}

	
	if (ov_open (f, &vf, NULL, 0) < 0) {
		debug_printf (INTERFACE, D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Input isn't ogg");
		g_thread_exit (0);
	}

	
	vi = ov_info (&vf, -1);
	format.bits = 16;	
	format.channels = vi->channels;
	format.rate = vi->rate;
	format.byte_format = AO_FMT_LITTLE;	

	
	ao_initialize ();

	switch (Lecture_Gconf_Int (INT_AUDIO_DEVICE)) {
	    case 0:
		    default_driver = ao_driver_id ("arts");
		    break;
	    case 1:
		    default_driver = ao_driver_id ("alsa");
		    break;
	    case 2:
		    default_driver = ao_driver_id ("alsa09");
		    break;
	    case 3:
		    default_driver = ao_driver_id ("esd");
		    break;
	    case 4:
		    default_driver = ao_driver_id ("irix");
		    break;
	    case 5:
		    default_driver = ao_driver_id ("oss");
		    break;
	    default:
		    default_driver = ao_driver_id ("null");
	}

	device = ao_open_live (default_driver, &format, NULL);
	if (device == NULL) {
		debug_printf (INTERFACE, D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Couldn't find default driver");
		g_thread_exit (0);
	}

	while (1) {
		if (Sound_Alarme_Active) {
			while (1) {
				ret = ov_read (&vf, pcmout, BUF_SIZE, 0, 2, 1, &current_section);
				if (ret == 0)
					break;
				ao_play (device, pcmout, ret);
			}
			ov_raw_seek (&vf, 0);
		}
		else
			sleep (1);
	}

	
	ao_close (device);
	ao_shutdown ();

	
	ov_clear (&vf);
	g_thread_exit (0);
}



void Initialisation_Sound_Alert ()
{
	Sound_Alarme_Active = FALSE;
	Creation_Thread (Thread_Sound_Alert,NULL);
}
