/* ***********************IMPORTANT *********************************************
 *  Copyright (C) 2004-2006 Thierry Lagarde <tlagarde@users.sourceforge.net>
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *
 *                      ********************
 *
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *
 *                      ********************
 *
 *
 * You are highly encouraged to send your changes to <tlagarde@users.sourceforge.net>
 * for possible incorporation into the main distribution.
 * By sending these changes to <tlagarde@users.sourceforge.net>,
 * it is assumed that you are offering <tlagarde@users.sourceforge.net>
 * the unlimited, non-exclusive right to reuse, modify, and relicense the code.
 *
 *
 *                      ********************
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 **************************************************************************************/
#include "Update.h"
#include "Thread.h"
#include "Debug.h"
#include "Animation.h"
#include "Gconf.h"
#include "main.h"
#include "Interface/Demarrage_Interface.h"

gchar    *Update_Fingerprinting (gchar * uri)
{
	GnomeVFSHandle *handle = NULL;
	GnomeVFSResult result;
	gchar     buffer[BUF_SIZE_UPDATE];
	gchar    *Texte = NULL;
	GnomeVFSFileSize bytes_read;
	gint      Taille_Texte;

	ERROR_if (uri == NULL);
	Taille_Texte = 1;
	Texte = malloc (sizeof (gchar));
	strcpy (Texte, "");
	ERROR_if (Texte == NULL);
	result = gnome_vfs_open (&handle, uri, GNOME_VFS_OPEN_READ);
	debug_printf (INTERFACE, D_ALL, __FILE__, __PRETTY_FUNCTION__, __LINE__, "result : %d", result);
	if (result != GNOME_VFS_OK) {
		debug_printf (INTERFACE, D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Unable to open URI, aborting\n");
		gnome_vfs_close (handle);
		return NULL;
	}
	while (result == GNOME_VFS_OK) {
		result = gnome_vfs_read (handle, buffer, BUF_SIZE_UPDATE, &bytes_read);
		buffer[bytes_read] = 0;
		if (bytes_read != 0) {
			Taille_Texte = Taille_Texte + bytes_read;
			Texte = realloc (Texte, (sizeof (gchar) * (Taille_Texte + 1)));
			strcat (Texte, buffer);
			debug_printf (INTERFACE, D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "(%d )%s", (int) bytes_read, buffer);
		}
	}
	gnome_vfs_close (handle);
	return Texte;
}


gpointer Thread_Update_Fingerprinting ()
{
	gchar    *Num_Version, *Fingerprinting, *Destination, *Deb_Version;
	gint      Version;
	FILE     *fdestination = NULL;

	
	Etat = UPDATE_THREAD_OK;
	if ((Num_Version = Update_Fingerprinting ("http://autoscan.free.fr/FingerPrinting/Version.php")) == NULL)
		g_thread_exit (0);
	
	Etat = UPDATE_VERSION_OK;
	
	Version = 0;
	Deb_Version = strchr(Num_Version,']');
	Deb_Version[0] = 0;
	Deb_Version = strchr(Num_Version,'[')+1;
	
	
	Version = atoi(Deb_Version);
	g_free (Num_Version);
	

	if (Lecture_Gconf_Int (INT_VERSION_SIGNATURE) != Version) {
		Etat = UPDATE_DOWNLOAD;
		if ((Fingerprinting = Update_Fingerprinting ("http://autoscan.free.fr/FingerPrinting/Finger_Printing.xml")) == NULL) {
			Etat = UPDATE_FIN;
			g_thread_exit (0);
		}
		
		Destination = g_strdup_printf ("%s/.AutoScan/Config/Finger_Printing.xml", Info_User->pw_dir);
		fdestination = fopen (Destination, "w");
		if (fdestination == NULL) {
			debug_printf (INTERFACE, D_WARN_USER, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Creating file: %s: %s", Destination, strerror (errno));
			g_thread_exit (0);
		}
		fputs (Fingerprinting, fdestination);
		fclose (fdestination);
		g_free (Fingerprinting);
		
		Enregistrement_Gconf_Entier (INT_VERSION_SIGNATURE, Version);
	}
	Etat = UPDATE_FIN;
	debug_printf (INTERFACE, D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Fin Thread Update   (%d)", Lecture_Gconf_Int (INT_VERSION_SIGNATURE));
	g_thread_exit (0);
}



void Initialisation_Update ()
{
	gint      Timeout;
	gchar    *Text;

	Etat = UPDATE_DEBUT;
	
	Creation_Thread (Thread_Update_Fingerprinting, NULL);
	
	while (Etat == UPDATE_DEBUT) {
		refresh ();
		sleep (1);
	}
	Timeout = 15;
	while ((Timeout != 0) && (Etat == UPDATE_THREAD_OK)) {
		Text = g_strdup_printf ("Updating (Fingerprinting)   please wait (%d)", Timeout);
		gtk_progress_bar_set_text (GTK_PROGRESS_BAR (progressbar_Demarrage), Text);
		refresh ();
		g_free (Text);
		sleep (1);
		Timeout--;
	}
	if (Etat == UPDATE_THREAD_OK) {	
		gtk_progress_bar_set_text (GTK_PROGRESS_BAR (progressbar_Demarrage), _("Updating (Fingerprinting)   FAILED"));
		refresh ();
		sleep (2);
		return;
	}

	while (Etat == UPDATE_VERSION_OK) {
		sleep (1);
	}
	gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progressbar_Demarrage), 0.8);
	if (Etat == UPDATE_DOWNLOAD) {
		Timeout = 15;
		while ((Timeout != 0) && (Etat == UPDATE_DOWNLOAD)) {
			Text = g_strdup_printf ("Download (Fingerprinting)   please wait (%d)", Timeout);
			gtk_progress_bar_set_text (GTK_PROGRESS_BAR (progressbar_Demarrage), Text);
			refresh ();
			g_free (Text);
			sleep (1);
			Timeout--;
		}

	}

	gtk_progress_bar_set_text (GTK_PROGRESS_BAR (progressbar_Demarrage), _("Updating (Fingerprinting)   OK"));
	refresh ();
	sleep (2);
}
