/* ***********************IMPORTANT *********************************************
 *  Copyright (C) 2004-2006 Thierry Lagarde <tlagarde@users.sourceforge.net>
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *
 *                      ********************
 *
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *
 *                      ********************
 *
 *
 * You are highly encouraged to send your changes to <tlagarde@users.sourceforge.net>
 * for possible incorporation into the main distribution.
 * By sending these changes to <tlagarde@users.sourceforge.net>,
 * it is assumed that you are offering <tlagarde@users.sourceforge.net>
 * the unlimited, non-exclusive right to reuse, modify, and relicense the code.
 *
 *
 *                      ********************
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 **************************************************************************************/
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include <libgnomevfs/gnome-vfs.h>

#include "Interface/Root_Interface.h"
#include "Interface/Bulle_Port_Interface.h"
#include "Interface/About_Interface.h"
#include "Interface/Settings_Interface.h"
#include "Interface/Add_Network_Interface.h"
#include "Interface/Menu_Samba_Interface.h"
#include "Interface/Copie_Fichier_Interface.h"
#include "Interface/Fingerprinting_Interface.h"
#include "Interface/Recherche_Interface.h"
#include "Interface/Save_Interface.h"
#include "Interface/Alert_Interface.h"
#include "Interface/Bulle_Network_Interface.h"
#include "Interface/Erreur_Interface.h"
#include "Interface/Debug_Interface.h"
#include "Interface/Demarrage_Interface.h"
#include "Reseau.h"
#include "Objet.h"
#include "Animation.h"
#include "Objet_Bulle.h"
#include "Modules.h"
#include "Gconf.h"
#include "Settings.h"
#include "Settings_Signature.h"
#include "Settings_Port.h"
#include "Settings_Script.h"
#include "Debug.h"
#include "Com/Communication.h"
#include "Add_Network_Subnet.h"
#include "Fournisseur.h"
#include "Equipement.h"
#include "Module_Samba.h"
#include "main.h"
#include "Style.h"
#include "Declaration_Constante.h"
#include "Conf.h"
#include "Thread.h"
#include "Sound.h"
#include "Applet.h"
#include "Settings_Restore.h"
#include "Quit.h"
#include "Restoration_Agent.h"
#include "Update.h"


extern struct Struct_Modele_Objet Liste_Modele_Objet[];


int main (int argc, char *argv[])
{
	int       ptimer_Reception_Donnee = 0, ptimer_Rachaichir_Info = 0;
	gchar    *Chemin;

#ifdef ENABLE_NLS
	bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
	textdomain (GETTEXT_PACKAGE);
#endif
	printf ("Starting %s (http://AutoScan.free.fr) %s\n", VERSION_AUTOSCAN, version_compiled);
	Pid_AutoScan = getpid ();
	
	g_thread_init (NULL);
	gdk_threads_init ();
	gtk_init (&argc, &argv);
	gnome_program_init (PACKAGE, VERSION, LIBGNOMEUI_MODULE, argc, argv, GNOME_PARAM_APP_DATADIR, PACKAGE_DATA_DIR, NULL);
	client_gconf = gconf_client_get_default ();
	gnome_vfs_init ();
	Initialisation_Style ();
	create_window_root ();
	create_win_Demarrage ();
	create_Debug ();
	create_window_Erreur ();
	Initialisation_Debug ();
	Initialisation_signaux ();

	
	creation_applet_icone ();

	
	Info_User = getpwuid (getuid ());
	Copie_Fichier_Configuration ();
	Initialisation_Variable_Reseau ();
	Initialisation_Settings ();
	Initialisation_Subnet ();
	Initialisation_Equipement ();
	Initialisation_Variable_Restoration_Agent ();

	
	Initialisation_Module_Samba ();
	Init_Liste_Partage_Monter ();

	
	gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progressbar_Demarrage), 0.1);
	gtk_progress_bar_set_text (GTK_PROGRESS_BAR (progressbar_Demarrage), _("Loading (Port) ...."));
	refresh ();
	Chemin = g_strdup_printf ("%s/.AutoScan/Config/Port.xml", Info_User->pw_dir);
	Lit_Xml_Port_Fichier (Chemin);
	g_free (Chemin);
	gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progressbar_Demarrage), 0.2);
	gtk_progress_bar_set_text (GTK_PROGRESS_BAR (progressbar_Demarrage), _("Loading (Subnet)"));
	refresh ();
	Chemin = g_strdup_printf ("%s/.AutoScan/Config/Subnet.xml", Info_User->pw_dir);
	Lit_Xml_Subnet_Fichier (Chemin);
	g_free (Chemin);
	gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progressbar_Demarrage), 0.3);
	gtk_progress_bar_set_text (GTK_PROGRESS_BAR (progressbar_Demarrage), _("Loading (Script)"));
	refresh ();
	Chemin = g_strdup_printf ("%s/.AutoScan/Config/Script.xml", Info_User->pw_dir);
	Lit_Xml_Script_Fichier (Chemin);
	g_free (Chemin);
	gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progressbar_Demarrage), 0.5);
	gtk_progress_bar_set_text (GTK_PROGRESS_BAR (progressbar_Demarrage), _("Loading (Restore)"));
	refresh ();
	Chemin = g_strdup_printf ("%s/.AutoScan/Config/Restore.xml", Info_User->pw_dir);
	Lit_Xml_Restore_Fichier (Chemin);
	g_free (Chemin);
	gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progressbar_Demarrage), 0.6);
	gtk_progress_bar_set_text (GTK_PROGRESS_BAR (progressbar_Demarrage), _("Loading (Mac)"));
	refresh ();
	Creer_Liste_Fournisseur ();
	gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progressbar_Demarrage), 0.7);
	gtk_progress_bar_set_text (GTK_PROGRESS_BAR (progressbar_Demarrage), _("Updating (Fingerprinting)   please wait"));
	refresh ();
	Initialisation_Update();

	gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progressbar_Demarrage), 0.9);
	gtk_progress_bar_set_text (GTK_PROGRESS_BAR (progressbar_Demarrage), _("Loading (Fingerprinting)"));
	refresh ();
	Chemin = g_strdup_printf ("%s/.AutoScan/Config/Finger_Printing.xml", Info_User->pw_dir);
	Lit_Xml_Signature_Fichier (Chemin);
	g_free (Chemin);

	
	Charge_Rachaichissement_Icone_Liste ();

	
	gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (progressbar_Demarrage), 0.9);
	gtk_progress_bar_set_text (GTK_PROGRESS_BAR (progressbar_Demarrage), _("Loading (Interface)"));
	refresh ();
	Initialisation_Modules ();
	gtk_window_set_policy (GTK_WINDOW (window_root), TRUE, TRUE, FALSE);	
	gtk_widget_realize (window_root);	
	create_window_Add_Network ();
	Afficher_Liste_Subnet ();
	if (!(Lecture_Gconf_Boolean (BOOL_MINIMIZED_TRAY_ICON) && (Lecture_Gconf_Boolean (BOOL_START_MINIMIZED)))) {
		gtk_widget_show (window_root);
		if (g_list_length (Settings->GList_Restore) == 0)
			gtk_widget_show (window_Add_Network);
	}
	create_dialog_Settings ();
	create_Wizard_Add_OS_Fingerprinting ();
	Afficher_Liste_Signature ();
	Afficher_Liste_Port ();
	Afficher_Liste_Script ();
	Afficher_Liste_Restore ();
	create_about ();
	create_win_Alert ();
	create_Menu_Samba ();
	create_window_Copie_Fichiers ();
	create_window_Recherche ();
	create_file_Save ();
	win_raccourci = create_Bulle_Port ();
	create_Bulle_Reseau ();
	gtk_widget_realize (win_raccourci);	
	Liste_Modele_Objet[PORTS].Win_Bulle = win_raccourci;


	
	Communication_Initialisation ();
	ptimer_Reception_Donnee = gtk_timeout_add (1000, Communication_Reception_Donnee_Boucle, NULL);
	ptimer_Rachaichir_Info = gtk_timeout_add (1000, Rachaichi_Info_Transfert_Entre_Agent, NULL);

	

	Restoration_Liste_Fichier ();



	
	ptimer_Rachaichissement_Liste = gtk_timeout_add (500, timer_Rachaichissement_Liste, NULL);
	Etat_Rafraichissement = 0;
	ptimer_Alerte_Liste = gtk_timeout_add (500, timer_Alerte_Liste, NULL);
	ptimer_Test_Etat_Liste = gtk_timeout_add (500, timer_Test_Etat_Liste, NULL);

	
	Initialisation_Sound_Alert ();

	
	
	gtk_widget_hide (win_Demarrage);
	gtk_main ();
	
	return 0;
}
