/* ***********************IMPORTANT *********************************************
 *  Copyright (C) 2004-2006 Thierry Lagarde <tlagarde@users.sourceforge.net>
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *
 *                      ********************
 *
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *
 *                      ********************
 *
 *
 * You are highly encouraged to send your changes to <tlagarde@users.sourceforge.net>
 * for possible incorporation into the main distribution.
 * By sending these changes to <tlagarde@users.sourceforge.net>,
 * it is assumed that you are offering <tlagarde@users.sourceforge.net>
 * the unlimited, non-exclusive right to reuse, modify, and relicense the code.
 *
 *
 *                      ********************
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 **************************************************************************************/
#include <glib.h>
#include <sys/msg.h>
#include <errno.h>
#include "Communication.h"
#include "Commun.h"
#include "../Debug.h"
#include "../Thread.h"
#include "../Intrus.h"
#include "../Notification_Port.h"
#include "Socket.h"
#include "Semaphore.h"
#include "Scan_Port.h"
#include "Scan_Signature.h"
#include "Sous_Reseau_Autorise.h"
#include "Initialisation_Com.h"
#include "Notification_Equipement.h"
#include "../Restoration.h"
#include "../Nessus.h"
#include "Declaration_Constante.h"
#include "../Scan_Equipement.h"


void Arret_Demande (gpointer * Message)
{
	debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "[S] Demande d'arret (AutoScan) Socket ouvert[%d]  Thread[%d]", Nb_Socket_Ouverte, Info_Demarrage.Nb_Thread_Max - Valeur_Semaphore (&Semaphore_Thread_Scan));
	printf("Stopping %s (http://AutoScan.free.fr) %s  pid(%d)\n", VERSION_AGENT, version_compiled,getpid());
	exit (0);
}



struct Struct_Modele_Communication Liste_Communication[] = {
	{AGENT_VERSION, NULL},
	{AJOUT_PORT, &Ajout_Un_Port},
	{AJOUT_SIGNATURE, NULL},	
	{RESTAURATION_MACHINE, &Ajout_Equipement_Restoration},
	{AJOUT_RESEAU_PRIVEE, &Ajout_Sous_Reseau_Autorise},
	{DEMANDE_LISTE_INTERFACE, &Envoie_Liste_Interface},
	{NOUVELLE_INTERFACE, NULL},
	{FIN_LISTE_INTERFACE, NULL},
	{DEMARRAGE_AGENT, &Demarrage_Agent},
	{DEMANDE_ARRET_AGENT, &Arret_Demande},
	{RETOUR_MESSAGE, NULL},
	{RETOUR_ERREUR_CRITIQUE, NULL},
	{INFO_AGENT, NULL},
	{NOUVEAU_EQUIPEMENT, NULL},
	{DEMANDE_NESSUS, &Initialisation_Nessus},
	{DEMARRAGE_NESSUS, NULL},
	{RETOUR_NESSUS_INFO, NULL},
	{RETOUR_NESSUS_DONNEE, NULL},
	{RETOUR_NESSUS_FIN, NULL},
	{DEMANDE_RAFRAICHIR_ETAT_EQUIMEMENT, &Rafraichir_Etat_Equipement},
	{RETOUR_RAFRAICHIR_ETAT_EQUIMEMENT, NULL},
	{DEMANDE_RAFRAICHIR_LISTE_PORTS, NULL},
	{RETOUR_RAFRAICHIR_LISTE_PORTS, NULL},
	{DEMANDE_RAFRAICHIR_ENREGISTREMENT_SAMBA, NULL},
	{RETOUR_RAFRAICHIR_ENREGISTREMENT_SAMBA, NULL},
	{DEMANDE_RAFRAICHIR_SIGNATURE, NULL},
	{RETOUR_RAFRAICHIR_SIGNATURE, NULL},
	{DEMANDE_NOTIFICATION_WARNING_ETAT_EQUIPEMENT, NULL},
	{DEMANDE_NOTIFICATION_WARNING_ETAT_PORT, NULL},
	{DEMANDE_NOTIFICATION_CRITIQUE_ETAT_EQUIPEMENT, &Ajout_Notification_Equipement},
	{DEMANDE_NOTIFICATION_CRITIQUE_ETAT_PORT, &Ajout_Notification_Port},
	{RETOUR_NOTIFICATION_CHANGEMENT_ETAT_EQUIPEMENT, NULL},
	{RETOUR_NOTIFICATION_CHANGEMENT_ETAT_PORT, NULL},
	{ANNULATION_NOTIFICATION_ETAT_EQUIPEMENT, &Enlever_Notification_Equipement},
	{ANNULATION_NOTIFICATION_ETAT_PORT, &Enlever_Notification_Port},
	{ACTIVE_ALERTE_INTRUS, &Active_Alerte_Intrusion},
	{DESACTIVE_ALERTE_INTRUS, &Desactive_Alerte_Intrusion},
	{ALERTE_INTRUS, NULL},
	{DEMANDE_MODIFIE_NIVEAU_DEBUG, NULL},
	{RETOUR_LISTE_SNMP,NULL},
	{DEMANDE_LANCEMENT_SCRIPT, NULL},
	{RETOUR_SCRIPT, NULL},
	{RETOUR_SCRIPT_TERMINER, NULL},
	{DEMANDE_RAFRAICHIR_EQUIPEMENT, &Demande_Rafraichir_Equipement},
	{RETOUR_RAFRAICHIR_EQUIPEMENT, NULL},
};


void     *Communication_Reception_Donnee_Boucle ()
{
	struct struct_Info Donnee;
	gint      Taille_Donnee_Recue;

	debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Communication_Reception_Donnee_Boucle    pid(%d)", Mon_Pid);
	while (1) {
		if ((Taille_Donnee_Recue = msgrcv (File_Info, (void *) &Donnee, sizeof (struct struct_Info) - sizeof (long), Mon_Pid, 0)) > -1) {
			if (Liste_Communication[Donnee.ID_Info].Gestion_Communication != NULL)
				(*Liste_Communication[Donnee.ID_Info].Gestion_Communication) ((gpointer *) & Donnee.Donnee);
			else
				debug_printf (D_WARN_USER, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Reception Message sans action defini");
		}
		if ((Taille_Donnee_Recue == -1) && (errno != EINTR))
			debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "%s", strerror (errno));
	}
}


void Communication_Initialisation ()
{
	Mon_Pid = getpid ();
	if ((File_Info = msgget (KEY_FILE, 0)) == -1)	
		debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "%s \nRun 'AutoScan'", strerror (errno));
	demmarrage_Thread (Communication_Reception_Donnee_Boucle, NULL);
}
